#pragma warning(disable:4996)
#include <windows.h>
#include <cstdio>
#include "tchar.h"
#include "resource.h"
#include <process.h>

#include "pipe_srv.h"

#define APPCLASS _T("ClientAppClass")
#define APPTITLE _T("ClientAppTitle")

LRESULT CALLBACK MainWndProc(HWND,UINT,WPARAM,LPARAM);
BOOL CALLBACK DialogProc(HWND,UINT,WPARAM,LPARAM);
void initialize(HWND hDlg,WCHAR*str);

HWND hDlgGlob;
static HWND hWndGlob;

// for pipe server
static pipe_srv_t pipe_data;

void HandleInstanceMessage(HWND AppWnd){
  HWND H;
  DWORD CurID, ThrID;
  bool R;
  LPDWORD lpdwProcessId=0;

  H = GetForegroundWindow();
  ThrID = GetWindowThreadProcessId(H, lpdwProcessId);
  CurID = GetCurrentThreadId();
  R = AttachThreadInput(CurID, ThrID, true);
  SetForegroundWindow(AppWnd);
  if(R)
   AttachThreadInput(CurID, ThrID, false);
}

int pipe_callback(std::wstring const&ini){
  WCHAR buff[2048]={0};
  wcsncpy(buff,ini.c_str(),2047);
  COPYDATASTRUCT data={0,(DWORD)(wcslen(buff)+1)*sizeof(WCHAR),(void*)(buff)};
  SendMessage(hDlgGlob,WM_COPYDATA,(WPARAM)NULL,(LPARAM)&data);
  return 0;
}

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow){
  HWND hWnd;
  WNDCLASS WndClass;
  MSG Msg;
  WCHAR szClassName[]=APPCLASS;
  WndClass.style=CS_HREDRAW|CS_VREDRAW;
  WndClass.lpfnWndProc=MainWndProc;
  WndClass.cbClsExtra=0;
  WndClass.cbWndExtra=0;
  WndClass.hInstance=hInstance;
  WndClass.hIcon=LoadIcon(NULL,IDI_APPLICATION);
  WndClass.hCursor=LoadCursor(NULL,IDC_ARROW);
  WndClass.hbrBackground=(HBRUSH)GetStockObject(WHITE_BRUSH);
  WndClass.lpszMenuName=NULL;
  WndClass.lpszClassName=szClassName;

  if(!RegisterClass(&WndClass)){
    MessageBox(NULL,_T("Can't register class"),_T("Error"),MB_OK);
    return 0;
  }

  WCHAR szWndTitle[]=APPTITLE;

  hWnd=CreateWindowEx(WS_EX_TOPMOST,szClassName,szWndTitle,
                    WS_OVERLAPPEDWINDOW,
                    CW_USEDEFAULT,CW_USEDEFAULT,
                    CW_USEDEFAULT,CW_USEDEFAULT,
                    NULL,NULL,
                    hInstance,NULL);
  if(!hWnd){
    MessageBox(NULL,_T("Can't create window"),_T("Error"),MB_OK);
    return 0;
  }
  hWndGlob=hWnd;

  HWND hDialog=CreateDialog(hInstance,
                            MAKEINTRESOURCE(IDD_DIALOG1),
                            0,(DLGPROC)DialogProc);
  
  hDlgGlob=hDialog;
  ShowWindow(hDialog,SW_SHOW);
  UpdateWindow(hDialog);

  int pNumArgs=0;
  PWSTR*ppArgv=CommandLineToArgvW(GetCommandLineW(),&pNumArgs);
  if(pNumArgs==2){
    COPYDATASTRUCT data={0,(DWORD)(wcslen(ppArgv[1])+1)*sizeof(WCHAR),(void*)(ppArgv[1])};
    SendMessage(hDialog,WM_COPYDATA,(WPARAM)NULL,(LPARAM)&data);
  }

  // setup pipe server
  HANDLE pipe_srv_thread=0;
  {
    pipe_data.ps_proc_=(work_proc_t)(&pipe_callback);// set callback
    pipe_data.ps_stop_=CreateEvent(NULL,TRUE,FALSE,NULL);// set terminate event
    unsigned int thread_id=0;
    errno=0;
    pipe_srv_thread=(HANDLE)_beginthreadex(NULL,0,pipe_srv_read_pipe_proc,(void*)&pipe_data,0,&thread_id);
    if(!pipe_srv_thread){
      ERROR_LOG("ClientSample::main(): failed to _beginthreadex().",errno,__LINE__,__FILE__);
      // kill our app because our app may launched yet (see "Pipe: Launch Client Application" regvalue)
      // or you may skip this logic
      goto shutdown;
    }
  }

  HeapFree(GetProcessHeap(),0,ppArgv);

  while(GetMessage(&Msg,NULL,0,0)){
    TranslateMessage(&Msg);
    DispatchMessage(&Msg);
  }

shutdown:
  // terminate pipe server
  SetEvent(pipe_data.ps_stop_);
  switch(WaitForSingleObject(pipe_srv_thread,5000)){
    case WAIT_OBJECT_0:
      break;
    default:
      TerminateThread(pipe_srv_thread,1);
  }

  CloseHandle(pipe_data.ps_stop_);

  return (int)Msg.wParam;
}

LRESULT CALLBACK MainWndProc(HWND hWnd,UINT Message,
                             WPARAM wParam,LPARAM lParam){

  switch(Message){
    case WM_COPYDATA:
      SendMessage(hDlgGlob,WM_COPYDATA,wParam,lParam);
      return 0;
    case WM_DESTROY:
      PostQuitMessage(0);
      return 0;
  }
  return DefWindowProc(hWnd,Message,wParam,lParam);
}

BOOL CALLBACK DialogProc(HWND hDlg,UINT Message,
                         WPARAM wParam,LPARAM lParam){

  PAINTSTRUCT ps;
  HDC hdc;
  static COPYDATASTRUCT*data=0;
  static HWND hwndList;
  switch(Message){
    case WM_INITDIALOG:
      HandleInstanceMessage(hDlg);
      SetWindowText(hDlg,_T("JOB-file content"));
      hwndList=GetDlgItem(hDlg,IDC_LIST1);
      initialize(hwndList,NULL);
      break;
    case WM_COMMAND:
      break;
    case WM_PAINT:
      hdc=BeginPaint(hDlg,&ps);
      EndPaint(hDlg,&ps);
      return 0;
    case WM_COPYDATA:
      data=(COPYDATASTRUCT*)lParam;
      initialize(hwndList,(WCHAR*)data->lpData);
      return 0;
    case WM_CLOSE:
      DestroyWindow(hDlg);
      break;
    case WM_DESTROY:
      PostQuitMessage(0);
      break;
  }
  return 0;
}

void initialize(HWND hList,WCHAR*str){
  if(!hList)
    return;
  SendMessage(hList,LB_RESETCONTENT,0,0);

  if(!str){
    SendMessage(hList,LB_ADDSTRING,0,(LPARAM)_T("Waiting for input file path..."));
    return;
  }

  WCHAR buff[512]={0};
  int pos=0;
  FILE*file=_wfopen(str,_T("rb"));
  while(fwscanf(file,_T("%511[^\n]\n"),buff)!=EOF)
    SendMessage(hList,LB_INSERTSTRING,(WPARAM)pos++,(LPARAM)buff);

  fclose(file);
  return;
}
