#include <windows.h>
#include <process.h>

#include "pipe_srv.h"

/**
 * simple one-thread named pipe server
 */
unsigned int __stdcall pipe_srv_read_pipe_proc(void*param){
  SECURITY_ATTRIBUTES sec_attr;
  SECURITY_DESCRIPTOR*sec_desc=(SECURITY_DESCRIPTOR*)LocalAlloc(LPTR,SECURITY_DESCRIPTOR_MIN_LENGTH);
  InitializeSecurityDescriptor(sec_desc,SECURITY_DESCRIPTOR_REVISION);
  SetSecurityDescriptorDacl(sec_desc,TRUE,(PACL)NULL,FALSE);
  sec_attr.nLength=sizeof(SECURITY_ATTRIBUTES);
  sec_attr.bInheritHandle=TRUE;
  sec_attr.lpSecurityDescriptor=sec_desc;

  pipe_srv_t*srv=(pipe_srv_t*)param;
  DWORD nbytes=0;

  WCHAR buff[PIPE_BUFF_SIZE]={0};
  int buff_size=PIPE_BUFF_SIZE*sizeof(WCHAR);

  OVERLAPPED overlap;
  memset(&overlap,0,sizeof(overlap));

  HANDLE overlap_event=CreateEvent(NULL,FALSE,TRUE,NULL);
  if(!overlap_event){
    ERROR_LOG("pipe_srv_read_pipe_proc(): failed to CreateEvent().",GetLastError(),__LINE__,__FILE__);
    _endthreadex(0);
    return 0;
  }

  overlap.hEvent=overlap_event;

  HANDLE events[2]={overlap_event,srv->ps_stop_};

  int received=0;

  HANDLE pipe=CreateNamedPipe(
    PIPE_NAME,                // pipe name 
    PIPE_ACCESS_DUPLEX|       // read/write access
    FILE_FLAG_OVERLAPPED,     // overlapped enable
    PIPE_TYPE_MESSAGE|        // message type pipe 
    PIPE_READMODE_MESSAGE|    // message-read mode 
    PIPE_WAIT,                // blocking mode 
    255,                      // max. instances  
    PIPE_BUFF_SIZE,           // output buffer size 
    PIPE_BUFF_SIZE,           // input buffer size 
    NMPWAIT_USE_DEFAULT_WAIT, // client time-out 
    &sec_attr);               // security attribute 

  /**
   * CreateNamedPipe will fail if you used "Pipe: Launch Client Application"==1 
   * and your app already started.
   */
  if(pipe==INVALID_HANDLE_VALUE){
    ERROR_LOG("pipe_srv_read_pipe_proc(): failed to CreateNamedPipe().",GetLastError(),__LINE__,__FILE__);
    _endthreadex(0);
    return 0;
  }

  for(;;){
    ConnectNamedPipe(pipe,&overlap);

    switch(WaitForMultipleObjects(2,events,FALSE,INFINITE)){
      case WAIT_OBJECT_0: // connect event
        if(!GetOverlappedResult(pipe,&overlap,&nbytes,FALSE)){
          ERROR_LOG("pipe_srv_read_pipe_proc(): failed to ConnectNamedPipe().",GetLastError(),__LINE__,__FILE__);
          goto skip;
        }
        break;
      case WAIT_OBJECT_0+1:// terminate event
        DisconnectNamedPipe(pipe);
        CloseHandle(pipe);
        CloseHandle(overlap_event);
        _endthreadex(0);
        return 0;

      default:
        ERROR_LOG("pipe_srv_read_pipe_proc(): failed to WaitForMultipleObjects().",GetLastError(),__LINE__,__FILE__);
        DisconnectNamedPipe(pipe);
        CloseHandle(pipe);
        CloseHandle(overlap_event);
        _endthreadex(0);
        return 0;
    }


    memset(buff,0,buff_size);

    memset(&overlap,0,sizeof(overlap));
    overlap.hEvent=overlap_event;

    ReadFile(pipe,buff,buff_size,&nbytes,&overlap);

    switch(WaitForMultipleObjects(2,events,FALSE,INFINITE)){
      case WAIT_OBJECT_0:// read event
        if(!GetOverlappedResult(pipe,&overlap,&nbytes,FALSE)){
          ERROR_LOG("pipe_srv_read_pipe_proc(): failed to ReadFile().",GetLastError(),__LINE__,__FILE__);
          goto skip;
        }
        buff[nbytes]=0;// buff store path to the ini-file

        // call users callback
        (*srv->ps_proc_)(buff);
        break;
      case WAIT_OBJECT_0+1:// terminate event
        DisconnectNamedPipe(pipe);
        CloseHandle(pipe);
        CloseHandle(overlap_event);
        _endthreadex(0);
        return 0;

      default:
        ERROR_LOG("pipe_srv_read_pipe_proc(): failed to WaitForMultipleObjects().",GetLastError(),__LINE__,__FILE__);
        DisconnectNamedPipe(pipe);
        CloseHandle(pipe);
        CloseHandle(overlap_event);
        _endthreadex(0);
        return 0;
    }

    skip:
      DisconnectNamedPipe(pipe);
  }

  _endthreadex(0);
  return 0;
}
